@@ -19,14 +19,15 @@ formats are generic enough to be used by any 2-D game.  Their purpose is
 to unify level editing.
 """
 
-__version__ = "0.2.1"
+__version__ = "0.3"
 
 
 import json
+import xml.etree.ElementTree as ET
 import warnings
 
 
-__all__ = ["ASCL", "JSL", "LevelObject"]
+__all__ = ["ASCL", "JSL", "ULX", "LevelObject"]
 
 
 class ASCL(object):
@@ -128,9 +129,10 @@ class JSL(object):
 
     - ``"meta"``: an object indicating the level's meta variables.
       These can be any kind of value.
-    - ``"objects"``: an object with the level's object IDs as keys. Each
-      value is an array of objects in the level with the corresponding
-      key, as an array with the following values:
+    - ``"objects"``: an object with the level's object types as keys.
+      Each value is an array of objects in the level of the respective
+      type, each individual object being an array with the following
+      values:
 
       - The horizontal position of the object in the level.
       - The vertical position of the object in the level.
@@ -184,6 +186,114 @@ class JSL(object):
         json.dump(data, f, indent=4)
 
 
+class ULX(object):
+
+    """
+    This class loads, stores, and saves Universal Level XML (ULX) files.
+    This format is based on XML and generally has one of the following
+    extensions: ".ulx", ".xml".
+
+    A ULX file contains a root tree with the name "level" containing the
+    following children:
+
+    - ``meta``: contains one element for each of the level's meta
+      variables.  Each element's tag indicates the name of the meta
+      variable, while its text indicates the value.
+    - ``objects``: contains ``object`` elements.  Each ``object``
+      element has the following attributes:
+
+      - ``"type"``: the object's type.
+      - ``"x"``: the horizontal position of the object in the level.
+      - ``"y"``: the vertical position of the object in the level.
+      - ``"option"`` (optional): an extra option for the object.
+
+    .. attribute:: meta
+
+       A dictionary of all of the level's meta variables.
+
+       .. note::
+
+          The meta variables can be any value, but when the ULX is
+          saved, all meta variables will be automatically converted to
+          strings, since XML does not support any other type.
+
+    .. attribute:: objects
+
+       A list of objects in the level as :class:`jsl.LevelObject`
+       objects.
+
+       .. note::
+
+          The extra options of the objects can be any value, but when
+          the ULX is saved, any options which are not either strings or
+          :const:`None` will be automatically converted to strings,
+          since XML does not support any other type.
+    """
+
+    def __init__(self):
+        self.meta = {}
+        self.objects = []
+
+    @classmethod
+    def load(cls, f):
+        """
+        Load the indicated file and return a :class:`ulvl.ULX` object.
+        """
+        self = cls()
+
+        tree = ET.parse(f)
+        root = tree.getroot()
+
+        for child in root:
+            if child.tag == "meta":
+                for meta in child:
+                    self.meta[meta.tag] = meta.text
+            elif child.tag == "objects":
+                for obj in child.findall("object"):
+                    type_ = obj.attrib.get("type")
+                    x = float(obj.attrib.get("x", "0"))
+                    y = float(obj.attrib.get("y", "0"))
+                    if x == int(x):
+                        x = int(x)
+                    if y == int(y):
+                        y = int(y)
+                    option = obj.attrib.get("option")
+                    self.objects.append(LevelObject(type_, x, y, option))
+
+        return self
+
+    def save(self, f):
+        """Save the object to the indicated file."""
+        root = ET.Element("level")
+
+        meta_elem = ET.Element("meta")
+        for i in self.meta:
+            elem = ET.Element(i)
+            elem.text = self.meta[i]
+            meta_elem.append(elem)
+
+        root.append(meta_elem)
+
+        objects_elem = ET.Element("objects")
+        for obj in self.objects:
+            if obj.x == int(obj.x):
+                obj.x = int(obj.x)
+            if obj.y == int(obj.y):
+                obj.y = int(obj.y)
+            attr = {"x": str(obj.x), "y": str(obj.y)}
+            if obj.type is not None:
+                attr["type"] = str(obj.type)
+            if obj.option is not None:
+                attr["option"] = str(obj.option)
+            elem = ET.Element("object", attrib=attr)
+            objects_elem.append(elem)
+
+        root.append(objects_elem)
+
+        tree = ET.ElementTree(root)
+        tree.write(f, xml_declaration=True)
+
+
 class LevelObject(object):
 
     """
